<?php


/**
 * @param $form
 *
 * @return mixed
 */
function connectid_order_subscribe_form($form) {

  // User needs to be logged in via ConnectId to subscribe.
//  if (!connectid_login_is_user_via_connectid()) {
//    $login_url = url('connectid/login', ['query' => drupal_get_destination()]);
//    $form['login_link'] = [
//      '#type' => 'fieldset',
//      '#title' => t("ConnectID Login"),
//      'link' => [
//        '#makrup' => t('Already have an account on ConnectID? <a href="@url">Click here to login</a> and continue with your order.', ['@url' => $login_url]),
//      ]
//    ];
//  }

  $available_coupons = _connectid_order_get_active_coupons_as_options();
  $preferred_coupon = filter_input(INPUT_GET, 'coupon_code', FILTER_SANITIZE_STRING);
  $form['product'] = [
    '#type'        => 'radios',
    '#title'       => t('Product'),
    '#description' => t('Select which product you are purchasing.'),
    '#required'    => TRUE,
    '#options'     => $available_coupons,
  ];

  if ($preferred_coupon && isset($available_coupons[$preferred_coupon])) {
    $form['product']['#type'] = 'hidden';
    $form['product']['#value'] = $preferred_coupon;
  }

  // Get default form and tweak it
  $form['billing_address'] = _connectid_order_get_address_fieldset();
  $form['billing_address']['#title'] = t('Billing address');
  $form['billing_address']['#description'] = t('Fill the address for invoicing purposes.');

  // Set required fields
  $required_elements = ['street', 'postalPlace', 'phone'];
  foreach ($required_elements as $item_key) {
    $form['billing_address'][$item_key]['#required'] = TRUE;
  }

  $form['billing_address']['email'] = [
    '#type'     => 'textfield',
    '#title'    => t('Email'),
    '#required' => TRUE,
  ];

  $form['separate_shipping_address'] = [
    '#type'  => 'checkbox',
    '#title' => t('Use different shipping address'),
  ];

  $form['shipping_address'] = _connectid_order_get_address_fieldset();
  $form['shipping_address']['#title'] = t('Shipping address');
  $form['shipping_address']['#description'] = t('Fill the address for shipping purposes.');

  $form['paymentMethod'] = [
    '#type'     => 'radios',
    '#title'    => t('Payment method'),
    '#options'  => _connectid_order_get_active_payment_types(),
    '#required' => TRUE,
  ];

  $form['actions'] = [
    '#type'   => 'actions',
    '#weight' => 50,
  ];

  $form['actions']['submit'] = [
    '#type'        => 'submit',
    '#value'       => t('Submit order'),
    '#description' => t('Verify and submit your order.'),
  ];

  return $form;
}


/**
 * @param $form
 * @param $form_state
 */
function connectid_order_subscribe_form_validate($form, &$form_state) {
  $input = &$form_state['values'];

  _connectid_order_subscribe_form_validate_address($form['billing_address'], $input['billing_address']);

  if ($input['separate_shipping_address']) {
    _connectid_order_subscribe_form_validate_address($form['shipping_address'], $input['shipping_address']);
  }
}

/**
 * @param $fieldset
 * @param $fieldset_values
 */
function _connectid_order_subscribe_form_validate_address(&$fieldset, $fieldset_values) {
  switch ($fieldset_values['recipient_type']) {
    case 'company':
      if (empty($fieldset_values['companyName'])) {
        form_error($fieldset['companyName'], t("Company name is required when selecting 'Company' as 'Recipient type'"));
      }
      break;

    case 'private':
    default:
      if (empty($fieldset_values['firstName'])) {
        form_error($fieldset['firstName'], t("First name is required when selecting 'Private' as 'Recipient type'"));
      }
      if (empty($fieldset_values['lastName'])) {
        form_error($fieldset['lastName'], t("Last name is required when selecting 'Private' as 'Recipient type'"));
      }
  }
}

function connectid_order_subscribe_form_submit($form, &$form_state) {
    connectid_order_subscribe_process($form_state['values']);
}

/**
 * Callback for page '/connectid/subscribe/complete'
 *
 * @throws \Exception
 */
function connectid_order_subscribe_complete_page_callback() {
    $form_data = _connectid_order_subscription_form_data_get();
    // If a user is missing not connected via ConnectID should not be allowed to land on this page.
    if (empty($form_data) || !connectid_login_is_user_via_connectid()) {
        $options = [];
        if (isset($_GET['destination'])) {
            $options['query']['destination'] = $_GET['destination'];
            unset($_GET['destination']);
        }
        drupal_goto('connectid/subscribe', $options);
    }

    connectid_order_subscribe_handle_complete($form_data);
}

/**
 * @param array $form_input
 *
 * @throws \Exception
 */
function connectid_order_subscribe_handle_complete(array $form_input) {
    $order = _connectid_order_build_order_from_input_form($form_input);
    $auth_token = connectid_login_get_access_token();
    $connectid = connectid_get_provider();

    if (!$connectid->submitApiOrder($auth_token, $order)) {
        drupal_set_message("We are sorry for the inconvenience, please wait a minute and try again. If the problem persists please contact the support.");
    }
    else {
        connectid_order_store_order($order);
        // FIXME: Should we do it here?
        _connectid_order_subscription_form_data_clear();

        $destination = $_GET['destination'] ?? '<front>';
        $returnUrl = url($destination, ['absolute' => TRUE]);
        $url = $connectid->getCompleteOrderUrl($order, $returnUrl, $returnUrl);

        unset($_GET['destination']);
        drupal_goto($url);
    }
}


/**
 * Generates a Order object based on the user input.
 * Note that little validation is done at this step.
 *
 * @param array $form_input
 *   Array of input values. Should ber validated and structured like connectid_order_subscribe_form()
 *
 * @return bool|\ConnectID\Api\DataModel\Order
 */
function _connectid_order_build_order_from_input_form(array $form_input) {
    $order = new \ConnectID\Api\DataModel\Order();

    $order->withOrderDate(\DateTimeImmutable::createFromFormat('U', REQUEST_TIME))
      ->withPaymentMethod($form_input['paymentMethod'])
      ->withCurrency('NOK');

    $billing_address = _connectid_order_build_address_from_input_form($form_input['billing_address']);
    $order->withPayer($billing_address);

    // TODO
    $all_coupons = variable_get('connectid_order_coupons', []);
    $coupon = $form_input['product'];
    $coupon_data = NULL;
    foreach ($all_coupons as $product_id => $coupon_list) {
        if (isset($coupon_list[$coupon])) {
            $coupon_data = unserialize($coupon_list[$coupon]);
            break;
        }
    }
    if (empty($coupon_data)) {
        return FALSE;
    }
    $coupon = \ConnectID\Api\DataModel\CouponType::create($coupon_data);

    $order->withOrderAmount($coupon->getCouponPrice());

    $order_line_alpha = new \ConnectID\Api\DataModel\OrderLine();

    $order_line_alpha->withQuantity(1);
    $order_line_alpha->withProductSpecType('coupon');

    $order_line_alpha->withProductSpecCode($coupon->getCouponCode());
    $order_line_alpha->withProductSpecNo($coupon->getCouponNumber());
    $order_line_alpha->withUnitPrice($coupon->getCouponPrice());

    if ($form_input['separate_shipping_address'] ?? FALSE) {
        $shipping_address = _connectid_order_build_address_from_input_form($form_input['shipping_address']);
        $order_line_alpha->withReceiver($shipping_address);
    }

    $context = [
      'order' => clone $order,
      'coupon' => clone $coupon,
      'source' => 'build_order_from_input_form',
    ];
    drupal_alter('connectid_order_order_line', $order_line_alpha, $context);
    $order->withOrderLine($order_line_alpha);

    return $order;
}


/**
 * @return array
 *  Default fieldset for ConnectID Address, no fields are marked as required.
 */
function _connectid_order_get_address_fieldset() {
    $fieldset = [
      '#type'        => 'fieldset',
      '#title'       => t('Address'),
      '#description' => t('Fill the address.'),
      '#tree'        => TRUE,
      '#collapsible' => FALSE,
      '#collapsed'   => FALSE,
      '#attached'    => [
        'library' => [
          ['system', 'drupal.collapse'],
        ],
      ],
    ];

    $fieldset['recipient_type'] = [
      '#type'    => 'radios',
      '#title'   => t('Recipient type'),
      '#options' => [
        'company' => t('Company'),
        'private' => t('Private'),
      ],
    ];

    $fieldset['companyName'] = [
      '#type'  => 'textfield',
      '#title' => t('Company'),
    ];

    $fieldset['firstName'] = [
      '#type'  => 'textfield',
      '#title' => t('First name'),
    ];

    $fieldset['lastName'] = [
      '#type'  => 'textfield',
      '#title' => t('Last name'),
    ];

    $fieldset['street'] = [
      '#type'     => 'textfield',
      '#title'    => t('Address'),
    ];

    $fieldset['postalCode'] = [
      '#type'             => 'textfield',
      '#title'            => t('Postal code'),
      '#element_validate' => ['element_validate_integer_positive'],
    ];

    $fieldset['postalPlace'] = [
      '#type'     => 'textfield',
      '#title'    => t('City/Area'),
    ];

    $fieldset['phone'] = [
      '#type'     => 'textfield',
      '#title'    => t('Telephone'),
    ];

    return $fieldset;
}

/**
 * Given a validated input from a form generated by _connectid_order_get_address_fieldset(),
 *  will return a fully populated Address object.
 * No validation is done, so it could generate an object not valid for ConnectID.
 *
 * @param array $form_input
 *
 * @return \ConnectID\Api\DataModel\Address
 */
function _connectid_order_build_address_from_input_form(array $form_input) {
  if (!empty($form_input['email'])) {
    $form_input['emails'] = [
      $form_input['email'],
    ];
    unset($form_input['email']);
  }

  if (!empty($form_input['phone'])) {
    $form_input['phoneNumbers'] = [
      $form_input['phone'],
    ];
    unset($form_input['phone']);
  }

  unset($form_input['recipient_type']);
  $address = \ConnectID\Api\DataModel\Address::create($form_input);

  return $address;
}
